
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
   A simple generic implementation of the Bag abstract data type
   that uses a linked list implementation with an iterator.
<p>
   Here is the public Bag interface.
      public boolean remove(T element);
      public int size();
      public boolean isEmpty();
      public void add(T element);
      public void add(T... elements);
      public void addAll(Container c);
      public boolean contains(T element);
      public int countOccurrences(T element);
*/
public class LinkedBag<T> extends Bag<T>
{
   /**
      This is an implementation of the Iterator interface
      that knows how to iterator over a LinkedBag.
   <p>
      Since the LinkedBag class does not expose any information
      about its internal implementation, this Iterator must
      be a nested class within the LinkedBag class so that the
      iterator object has access to the internals of a LinkedBag
      object.
   */
   class LinkedBagIterator implements Iterator<T>
   {
      Node<T> cursor;

      /**

      */
      public LinkedBagIterator( )
      {
         // Here is where we make use of the fact that
         // this iterator is nested inside of the LinkedBag
         // class. We have access to the private field
         // first.
         this.cursor = first;
      }

      /**

      */
      public boolean hasNext()
      {
         return (this.cursor != null);
      }

      /**

      */
      public T next()
      {
         if ( hasNext() )
         {
            T temp = this.cursor.data;
            cursor = this.cursor.next;
            return temp;
         }
         else
            throw new NoSuchElementException();
      }

      /**

      */
      public void remove()
      {
         throw new UnsupportedOperationException();
      }
   }//LinkedBagIterator


   // A private field in the LinkedList class.
   private Node<T> first;

   /**

   */
   public LinkedBag()
   {
      super();
      first = null;
   }

   /**

   */
   public void add(T element)
   {
      Node<T> node = new Node<T>(element, first);
      first = node;
   }

   /**

   */
   public boolean remove(T element)
   {
      boolean result = false;

      if ( null == first )
      {
         result = false;
      }
      else if ( first.data.equals(element) )
      {
         first = first.next;
         size--;
         result = true;
      }
      else
      {
         Node<T> cursor = first;
         // find element in the list
         while ( (cursor.next != null) && (! cursor.next.data.equals(element)) )
         {
            cursor = cursor.next;
         }
         if (null == cursor.next)
         {
            result = false;
         }
         else
         {
            cursor.next = cursor.next.next;
            size--;
            result = true;
         }
      }
      return result;
   }


   // Provide an implementation of the Iterable interface.
   public Iterator<T> iterator()
   {
      return new LinkedBagIterator( );
   }

}//LinkedBag
