
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
   A simple generic implementation of the List abstract data type
   that uses a linked list implementation with an iterator.
<p>
   Here is the public List interface.
      public T getFirst();
      public T remove();
      public int size();
      public boolean isEmpty();
      public void add(T element);
      public void add(T... elements);
      public void addAll(Container c);
      public boolean contains(T element);
      public int countOccurrences(T element);
*/
public class LinkedList<T> extends List<T>
{
   /**
      This is an implementation of the Iterator interface
      that knows how to iterator over a LinkedList.
   <p>
      Since the LinkedList class does not expose any information
      about its internal implementation, this Iterator must
      be a nested class within the LinkedList class so that the
      iterator object has access to the internals of a LinkedList
      object.
   */
   class LinkedListIterator implements Iterator<T>
   {
      Node<T> cursor;

      /**

      */
      public LinkedListIterator( )
      {
         // Here is where we make use of the fact that
         // this iterator is nested inside of the LinkedList
         // class. We have access to the private field
         // first.
         this.cursor = first;
      }

      /**

      */
      public boolean hasNext()
      {
         return (this.cursor != null);
      }


      /**

      */
      public T next()
      {
         if ( hasNext() )
         {
            T temp = this.cursor.data;
            this.cursor = this.cursor.next;
            return temp;
         }
         else
            throw new NoSuchElementException();
      }

      public void remove()
      {
         throw new UnsupportedOperationException();
      }
   }//LinkedListIterator


   // A private field in the LinkedList class.
   private Node<T> first;


   /**

   */
   public LinkedList()
   {
      super();
      first = null;
   }


   /**

   */
   public T getFirst()
   {
      return first.data;
   }


   /**

   */
   public void add(T element)
   {
      Node<T> node = new Node<T>(element, first);
      first = node;
      size++;
   }


   /**

   */
   public T remove()
   {
      T data = first.data;
      first = first.next;
      size--;
      return data;
   }


   // Provide an implementation of the Iterable interface.
   public Iterator<T> iterator()
   {
      return new LinkedListIterator();
   }

}//LinkedList
